import java.util.*;
import org.bleb.paperboy.plugin.*;
import org.bleb.paperboy.web.*;
import org.bleb.paperboy.util.*;

/**
 * A plugin for the test data which assumes it will be accessible from
 * the URL http://localhost:8080/paperboy/...
 *
 * @author Andrew Flegg
 * @version 0.04 (07-Mar-2000)
 **/
public class PluginTest extends Plugin
{
    /**
     * Grab the specified sections and update the list of stories
     * which may be of interest to the user
     *
     * @param section String[] of keys from <code>sections</code>
     **/
    public void start(OptionConfig conf) throws Exception
    {
        OptionGroup url   = (OptionGroup)conf.getItem("url");
        String      HOST  = (String)url.getItem("host").getValue() +
            (String)url.getItem("path").getValue();

        Webpage     site  = new Webpage();
        HTMLBlock   index = new HTMLBlock();
        Enumeration enum  = ((Vector)conf.getValue()).elements();
        Story       current;
        this.stories       = new Vector();

        while (enum.hasMoreElements()) {
            Option o = (Option)enum.nextElement();
            if ( !(o instanceof OptionBoolean) ) continue;
            OptionBoolean i = (OptionBoolean)o;

            if ( ((Boolean)i.getValue()).booleanValue()) {
                // -- Get the index -------------------------------------------
                site.setURL(HOST + "/" + i.getName() + "/");
                site.connect();
                index.setString(site.getContent());
                site.close();

                // -- Process the index ---------------------------------------
                index.crop("<!--stories-->", "<!--/stories-->", false);
                do {
                    current = new Story(HOST +
                                        index.getNextTag("story").get("url"));
                    current.setSection ( "Test : " + i.getLabel() );
                    
                    index.removeStart();
                    index.moveToIgnoreCase("</story>", true);
                    current.setHeadline( index.removeStart() );

                    stories.add(current);
                } while (index.containsIgnoreCase("<story"));
            }
        }
        return;
    }


    /**
     * Fetch the rest of the Story so we've got all the content.
     *
     * @param s Story to update
     **/
    public void fetch(Story s) throws Exception
    {
        HTMLBlock page = new HTMLBlock(s.getData());

        page.crop("<!--story-->", "<!--/story-->", false);
        page.moveToIgnoreCase("<p>", true);
        s.setSummary( page.removeStart() );
        s.setData( page.toString() );

        return;
    }
}










