import java.util.*;
import org.bleb.paperboy.plugin.*;
import org.bleb.paperboy.web.*;
import org.bleb.paperboy.util.*;

/**
 * Plugin for <a href="http://www.wired.com/news/">Wired News</a>
 * which provides US-centric technology, business and culture
 * news. Updated regularly throughout the day it often provides a
 * half-way house between non-technical sites like <a
 * href="http://news.bbc.co.uk/">BBC Online</a> and &quot;nerd&quot;
 * sites, such as <a href="http://slashdot.org/">Slashdot</a>.
 *
 * @author Andrew Flegg
 * @version 0.94 (07-Mar-2000)
 **/
public class PluginWired extends Plugin
{
    private static final String HOST   = "http://www.wired.com";

    public void start(OptionConfig conf) throws Exception
    {
        Webpage     site  = new Webpage();
        HTMLBlock   index = new HTMLBlock();
        Enumeration enum  = ((Vector)conf.getItem( "sections"
                                                   ).getValue()).elements();
        Story       current;
        this.stories      = new Vector();

        while (enum.hasMoreElements()) {
            OptionBoolean i = (OptionBoolean)enum.nextElement();
            if ( ((Boolean)i.getValue()).booleanValue()) {
                site.setURL(HOST + "/news/" + i.getName() + "/");
                site.connect();
                index.setString(site.getContent());
                site.close();
        
                index.crop("<!-- begin generic desk -->",
                           "<!-- end generic desk -->",
                           false);
                  
                do {
                    current = new Story(HOST +
                                        index.getNextTag("a").get("href"));
                    current.setSection( "Wired : " + i.getLabel() );
                
                    index.removeStart();
                    index.moveToIgnoreCase("</a>", true);
                    current.setHeadline( index.removeStart() );
                    
                    index.moveToIgnoreCase("<font", false);
                    current.setSummary( index.getTagsSelection("<font",
                                                               "</font>",
                                                               false)
                                        );
                    stories.add(current);
                } while (index.containsIgnoreCase("<a "));
            }
        }
                
        return;
    }
    
    
    public void fetch(Story s) throws Exception
    {
        HTMLBlock page = new HTMLBlock(s.getData());
        Webpage   site = new Webpage();
        
        s.setURL( HOST + page.findURL("/news/print/") );
        site.setURL( s.getURL() );
        site.connect();                         // Get page proper
        page.setString( site.getContent() );
        site.close();
        
        page.crop("<!-- story -->", "<!-- end story -->", false);
        page.moveToIgnoreCase("<br>", false);
        page.moveToIgnoreCase("<br>", false);   // Skip past author stuff
        page.removeStart();
        
        if (page.containsIgnoreCase("<p> <center> <hr")) {
            page.removeBlock("<p> <center> <hr", "</center>", true);
        }
        
        page.resetPosition();
        while (page.replace("<pagebreak>", ""));

        page.resetPosition();
        while (page.replace(" color=\"#000000\"", ""));

        s.setData( page.toString() );
        s.setLastModified( (new Date()).getTime() );
        return;
    }
}
 
