package org.bleb.paperboy;

import java.io.*;
import java.util.*;
import org.bleb.paperboy.plugin.*;
import org.bleb.paperboy.web.*;
import org.bleb.paperboy.util.*;
import org.bleb.paperboy.rule.*;

/**
 * Get the news, ie. this implements "Gather mode".
 *
 * @version 0.30 (09-Mar-2000)
 **/
public class GetNews
{
    public static void main(String argv[])
    {
        Option.setCreateGUI(false);
        Config.init();
        Plugin.init();
        Cache.init();
        Rule.init();

        GetNews foo = new GetNews();
        foo.main();
        System.exit(0);
        return;
    }


    /**
     * Do the work
     *
     * @return exit code (0 = success)
     **/
    public int main()
    {
        // -- Set the web proxy
        //
        String host = Config.getProxyHost();
        if (! host.equals(""))
            Webpage.setProxy(host, Config.getProxyPort());

        // -- Process specified plugins
        //
        Plugin      current = null;
        RuleConfig  conf    = new RuleConfig();
        Enumeration enum    = Plugin.list();
        while (enum.hasMoreElements()) {
            DEBUG.print("GetNews.main: Running gc... ");
            System.gc();
            DEBUG.println("complete.");

            String pluginName = (String)enum.nextElement();
            Rule[] rules      = Rule.getList(pluginName);
            
            if ((rules != null) && (rules.length >= 1)) {
                DEBUG.println("GetNews.main: Processing " + pluginName);
                try {
                    current = (Plugin)Class.forName(pluginName).newInstance();
                    current.start( Plugin.getConfig(pluginName) );
                    updateList(current, Rule.getList(pluginName), conf);
                } catch(Exception e) {
                    DEBUG.println("GetNews.main: Failed - "+e.getMessage());
                    e.printStackTrace();
                }
            } else {
                DEBUG.println("GetNews.main: No rules for " + pluginName);
            }
        }

        // -- Write index file
        //
        FileOutput.Line out = new FileOutput.Line("index.html");
        if (! out.open()) {
            DEBUG.println("GetNews.main: Unable to continue without index.");
            return 1;
        }
        
        Story s;
        Enumeration list = Cache.list("html").elements();
        while (list.hasMoreElements()) {
            s = (Story)Cache.get( (String)list.nextElement() );
            out.print("<p><b><a href=\"" + s.getURL() + "\">" +
                      s.getHeadline() + "</a></b>&nbsp; &nbsp; <i>[ " +
                      s.getSection() + " ]</i>&nbsp; - &nbsp;" +
                      new Date(s.getLastModified()).toString() + "<br>\n" +
                      s.getSummary() + "\n" +
                      "<right><small>(<a href=\"" +
                      s.getOriginalURL() + "\">Original</a>)</small>" +
                      "</right></p>\n\n");
        }
        out.close();

        DEBUG.println("GetNews.main: Finished.");
        return 0;
    }


    private static void updateList(Plugin p, Rule[] rule, RuleConfig c)
    {
        Story       s = null;
        Enumeration list;

        // -- Fetch every story that isn't already cached and matches
        //    at least one of the rules
        //
        list = p.stories.elements();
        while (list.hasMoreElements()) {
            s = (Story)list.nextElement();
            if (! Cache.stored(s)) {

                // -- Look for a rule that matches
                //
                boolean m = false;
                for (int i=0; i<rule.length; i++) {
                    if (rule[i] == null) break;

                    String hd = s.getHeadline();  // Match against headline
                    String sm = s.getSummary();   // and summary
                    String bd = s.getData();      // and body if available

                    if (hd != null) m |= rule[i].matches(hd.toLowerCase(), c);
                    if (sm != null) m |= rule[i].matches(sm.toLowerCase(), c);
                    if (bd != null) m |= rule[i].matches(bd.toLowerCase(), c);
                    if (m) {
                        DEBUG.println("GetNews.updateList: Match - '" +
                                      rule[i].toString() + "'");
                        break;
                    }
                }

                // -- Having found match fetch the story
                //
                if (m) {
                    try {
                        s.update();    // Update body of story
                        p.fetch(s);    // Process body
                        s.store();     // Store updated story
                        DEBUG.println("GetNews.updateList: Got \"" +
                                           s.getHeadline() + "\"");
                    } catch (Exception e) {
                        DEBUG.println("GetNews.updateList: Failed on \"" +
                                      s.getHeadline() + "\" - " +
                                      e.getMessage());
                    }
                } else {
                    DEBUG.println("GetNews.updateList: No match for \"" +
                                  s.getHeadline() + "\"");
                }
            } else {
                DEBUG.println("GetNews.updateList: Skipping \"" + 
                              s.getHeadline() + "\"");
            }
        }
        return;
    }
}








