package org.bleb.paperboy;

import org.bleb.paperboy.plugin.*;
import org.bleb.paperboy.rule.*;
import org.bleb.paperboy.util.*;
import org.bleb.paperboy.gui.DisplayNews;
import java.util.*;

/**
 * The main wrapper for <i>Paperboy: Collection, Processing and
 * Presentation of Online News</i>. This class either starts the
 * GetNews class (for &quot;gather&quot; mode) or DisplayNews
 * (which uses a Swing GUI to show the collected stories.
 *
 * @author Andrew Flegg &lt;<a
 * href="mailto:andrew@bleb.org">andrew@bleb.org</a>&gt;
 * @version 1.10 (17-Sep-2000)
 **/
public class Paperboy
{
    private final static String IDENT  = "Paperboy - http://www.bleb.org/";
    public final static String VERSION = "1.10 (17-Sep-2000)";

    public static void main(String argv[])
    {
        // -- Find the type of command, default "display"
        //
        String action = "display";
        if (argv.length == 1) {
            action = argv[0];
        } else  if (argv.length > 1) {
            action = "h";
        }
        while (action.startsWith("-")) action = action.substring(1);

        // -- Do the appropriate action
        //
        if (action.equals("d") || action.equals("display")) {
            Option.setCreateGUI(true);
            Config.init();
            Plugin.init();
            Cache.init();
            Rule.init();
            DisplayNews display = new DisplayNews();

            display.main();
            return;
            
        } else if (action.equals("g") || action.equals("gather")) {
            Option.setCreateGUI(false);
            Config.init();
            Plugin.init();
            Cache.init();
            Rule.init();
            GetNews gather = new GetNews();

            gather.main();
            System.exit(0);
        }

        // -- If neither option, print the help
        //
        System.out.print("Paperboy - Collection, Processing " +
                         "and Presentation of Online News\n" +
                         "~~~~~~~~   v" + VERSION + "\n\n" +
                         "Usage: paperboy [OPTION]\n" +
                         "   -d, --display    open the main window: "+
                         "show results and configuration\n" +
                         "   -g, --gather     fetch pending stories\n"+
                         "   -h, --help       this information\n\n"+
                         "If no other option is specified, display is "+
                         "assumed.\nReport bugs to "+
                         "<andrew@bleb.org>\n\n");
        return;
    }
}
            
