package org.bleb.paperboy.gui;

import org.bleb.paperboy.gui.htmlview.*;
import org.bleb.paperboy.plugin.*;
import org.bleb.paperboy.util.*;
import org.bleb.paperboy.rule.*;
import org.bleb.paperboy.web.*;
import org.bleb.paperboy.*;
import javax.swing.text.html.HTMLDocument;
import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.event.*;
import java.net.URL;
import javax.swing.*;
import java.util.*;
import java.awt.*;

/**
 * Display the main GUI for the program
 *
 * @version 0.00 (09-Mar-2000)
 **/
public class DisplayNews extends Object
    implements ListSelectionListener,
               StoryDisplay
{
    protected JTable      list;       // Story list at top of window
    protected JEditorPane jep;        // Displays the current story
    protected JScrollPane jsp;        // Allow the story to scroll
    protected CacheLink   dataModel;  // Link to the Cache
    protected ActionDoer  actions;    // Monitor mouse actions

    protected JFrame             win;
    protected JToolBar           tools;
    protected JSplitPane         main;
    protected JPanel             view;
    protected JScrollPane        listView;
    protected JLabel             status;
    protected SimpleLinkListener sll;
    protected MiscConfigFrame    miscConf;
    protected PluginConfigFrame  pluginConf;

    protected final static String dataDir = System.getProperty("paperboy.data");


    // -- Quick wrapper so it can be started as an app -------------
    //
    public static void main(String argv[]) throws Exception
    {
        Option.setCreateGUI(true);
        Config.init();
        Plugin.init();
        Cache.init();
        Rule.init();

        DisplayNews foo = new DisplayNews();
        foo.main();
        return;
    }


    /**
     * Open the main application's window
     *
     * @return Exit code for the app
     **/
    public int main()
    {
        // -- Initialisation stuff
        DEBUG.println("DisplayNews.main: Creating GUI...");

        // -- Create the objects for the window and the window itself
        win        = new JFrame("Paperboy");
        tools      = new JToolBar();
        main       = new JSplitPane();
        view       = new JPanel();
        list       = new JTable();
        listView   = new JScrollPane(list);
        jep        = new JEditorPane();
        jsp        = new JScrollPane(jep);
        status     = new JLabel(" ");
        sll        = new SimpleLinkListener(jep, null, status);
        actions    = new ActionDoer(win);
        miscConf   = new MiscConfigFrame();
        pluginConf = new PluginConfigFrame();

        // -- Create the menus
        DEBUG.println("DisplayNews.main: Creating menus...");
        JMenu file = new JMenu("File");
        file.setMnemonic('F');
        file.add(actions.menuItem("Get news...", "gather", 'G', 0));
        file.add(actions.menuItem("Exit", "exit", 'X', KeyEvent.VK_Q));

        JMenu edit = new JMenu("Edit");
        edit.setMnemonic('E');
        edit.add(actions.menuItem("Copy", "copy", 'C', KeyEvent.VK_C));
        edit.add(actions.menuItem("Select all", "select", 'A', KeyEvent.VK_A));

        JMenu       viewMenu = new JMenu("View");
        ButtonGroup showGrp  = new ButtonGroup();
        viewMenu.setMnemonic('V');
        viewMenu.add(actions.menuRadioItem("Show all", "showall", showGrp));
        viewMenu.add(actions.menuRadioItem("Show unread", "shownew", showGrp));
        viewMenu.addSeparator();
        viewMenu.add(actions.menuItem("Options...", "options", 'O', 0));
        viewMenu.add(actions.menuItem("Plugins...", "plugins", 'P', 0));

        JMenu story = new JMenu("Story");
        story.setMnemonic('S');
        story.add(actions.menuItem("Next", "next", 'N', KeyEvent.VK_N));
        story.add(actions.menuItem("Previous", "prev", 'P', KeyEvent.VK_P));
        story.addSeparator();
        story.add(actions.menuItem("Open original...", "original", 'G', 0));
        story.add(actions.menuItem("Mark as unread", "unread", 'R', 0));
        story.add(actions.menuItem("Delete", "delete", 'D', KeyEvent.VK_D));

        JMenu help = new JMenu("Help");
        help.add(actions.menuItem("Homepage", "web", 'H', 0));
        help.addSeparator();
        help.add(actions.menuItem("About...", "about", 'A', KeyEvent.VK_F1));
        
        JMenuBar menu = new JMenuBar();
        menu.add(file);
        menu.add(edit);
        menu.add(viewMenu);
        menu.add(story);
        menu.add(help);
        win.setJMenuBar(menu);

        // -- Setup the toolbar buttons
        DEBUG.println("DisplayNews.main: Creating toolbar...");
        tools.putClientProperty("JToolBar.isRollover", Boolean.TRUE);
        tools.add(actions.toolItem("Next_up.gif",
                                   "Next_sel.gif",
                                   "Next_dep.gif", "next", KeyEvent.VK_N));
        tools.add(actions.toolItem("Prev_up.gif",
                                   "Prev_sel.gif",
                                   "Prev_dep.gif", "prev", KeyEvent.VK_P));
        tools.add(actions.toolItem("Delete_up.gif",
                                   "Delete_sel.gif",
                                   "Delete_dep.gif", "delete", 0));
        tools.addSeparator();
        tools.add(actions.toolItem("Copy_up.gif",
                                   "Copy_sel.gif",
                                   "Copy_dep.gif", "copy", 0));
        tools.add(actions.toolItem("Original_up.gif",
                                   "Original_sel.gif",
                                   "Original_dep.gif", "original", 0));

        // -- Configure the objects
        DEBUG.println("DisplayNews.main: Configuring objects...");
        win.getContentPane().setLayout(new BorderLayout());
        main.setOrientation(JSplitPane.VERTICAL_SPLIT);
        view.setLayout(new BorderLayout());

        jep.setEditable(false);
        jep.addHyperlinkListener(sll);
        jep.putClientProperty("paperboy.display", this);
        jep.setEditorKitForContentType("text/html",
                                       new PatchedHTMLEditorKit());

        view.add(jsp,    "Center");
        view.add(status, "South");
        main.setLeftComponent(listView);
        main.setRightComponent(view);

        win.getContentPane().add(tools, "North");
        win.getContentPane().add(main,  "Center");

        // -- Disable actions which depend on a story being shown
        actions.setEnabled("next", false);
        actions.setEnabled("prev", false);
        actions.setEnabled("delete", false);
        actions.setEnabled("copy", false);
        actions.setEnabled("original", false);
        actions.setEnabled("unread", false);

        // -- Set window size and close handler
        win.setSize(640, 512);
        win.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent we) {
                actions.actionPerformed(new ActionEvent(this, 0, "exit"));
            }
        });

        // -- Start doing the interesting stuff
        dataModel = new CacheLink();
        list.setModel( (TableModel)dataModel );
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        list.getSelectionModel().addListSelectionListener(this);

        list.setShowGrid(false);
        TableColumnModel t = list.getColumnModel();
        t.getColumn( t.getColumnIndex("New") ).setPreferredWidth(20);
        t.getColumn( t.getColumnIndex("Site") ).setPreferredWidth(140);
        t.getColumn( t.getColumnIndex("Headline") ).setPreferredWidth(320);
        t.getColumn( t.getColumnIndex("Date") ).setPreferredWidth(64);

        main.setDividerLocation( 120 );
        win.setVisible(true);

        return 0;
    }

    
    /**
     * Value has changed in the list, update the story pane
     *
     * @param e Which row was selected...?
     **/
    public void valueChanged(ListSelectionEvent e) {
        if (e.getValueIsAdjusting())
            return;
        
        ListSelectionModel lsm = (ListSelectionModel)e.getSource();

        if (lsm.isSelectionEmpty()) {
            jep.setText(null);
        } else {
            Story s = dataModel.getStory( lsm.getMinSelectionIndex() );
            displayStory(s.getOriginalURL());
        }
        return;
    }


    /**
     * Implements the StoryDisplay interface. Ensure the correct
     * story is displayed and selected. The original URL of the
     * story is passed, if it is cached locally it will be displayed
     * otherwise an external viewer program is used.
     *
     * @param u Original URL of the story to display
     **/
    public void displayStory(String u)
    {
        // -- Display a remote page if it isn't cached
        //
        if (! Cache.stored(u)) {
            openExternal(u);
            return;
        }

        // -- Display a page in our mini-browser
        //
        try {
            int entry = dataModel.getStoryIndex(u);
            int max   = dataModel.getRowCount() - 1;
            // -- Disable buttons if appropriate
            actions.setEnabled("prev", (entry != 0));
            actions.setEnabled("next", (entry != max));
            actions.setEnabled("delete", true);
            actions.setEnabled("original", true);
            actions.setEnabled("copy", true);
            actions.setEnabled("unread", true);

            jep.setPage("file:"+dataDir+"/" + Cache.localURL(u));
            dataModel.getStory(entry).setIsRead(Boolean.TRUE);
            list.getSelectionModel().setSelectionInterval(entry, entry);
        } catch (Exception e) {
            DEBUG.println("DisplayNews.displayStory: Couldn't open " + u +
                          " - " + e.getMessage());
        }
        return;
    }


    /**
     * Open a URL in the configured external viewer program.
     *
     * @param u URL to open
     **/
    protected void openExternal(String u)
    {
        try {
            String cmd = Config.getViewerCommand(u);
            DEBUG.println("DisplayNews.openExternal: Executing - " + cmd);
            Runtime.getRuntime().exec(cmd);
        } catch (java.io.IOException e) {
            DEBUG.println("DisplayNews.openExternal: Couldn't execute - " +
                          e.getMessage());
        }
        return;
    }


    // -----------------------------------------------------------------


    /**
     * Implement an interface to the various buttons and menu items
     *
     * @version 0.01 (05-Mar-2000)
     **/
    public class ActionDoer implements ActionListener
    {
        protected JRootPane tlp;     // Top Level Pane
        protected Hashtable items;   // Menu items/buttons to keep track of

        /**
         * Specify the top level window.
         *
         * @param tlw the top-level window of the app
         **/
        public ActionDoer(JFrame tlw)
        {
            super();
            tlp   = tlw.getRootPane();
            items = new Hashtable();
        }


        /**
         * Create toolbar icons easily.
         *
         * @param icon_norm Name of icon to use for <i>normal</i> position
         * @param icon_up Name of icon to use when mouse pointer over button
         * @param icon_pressed Name of icon to use when button has been clicked
         * @param command Command text passed to <code>actionPerformed()</code>
         * @param accel Key which will trigger this event when pressed
         * @return JButton suitable for adding to a <var>JToolBar</var>
         **/
        public JButton toolItem(String icon_norm,
                                String icon_up,
                                String icon_pressed,
                                String command,
                                int accel)
        {
            String iconBase = (String)System.getProperty("paperboy.lib");
            if (iconBase == null) {
                iconBase = "";
            } else {
                iconBase += "/gfx/";
            }

            JButton foo = new JButton(new ImageIcon(iconBase + icon_norm));
            foo.addActionListener(actions);
            foo.setBorder(null);
            foo.setBorderPainted(false);
            foo.setFocusPainted(false);
            foo.setRolloverEnabled(true);
            foo.setRolloverIcon(new ImageIcon(iconBase + icon_up));
            foo.setPressedIcon(new ImageIcon(iconBase + icon_pressed));
            foo.setActionCommand(command);
            if (accel != 0) {
                int mask = Event.CTRL_MASK;
                mask = 0;

                tlp.registerKeyboardAction(this, command,
                                           KeyStroke.getKeyStroke(accel, mask),
                                           JComponent.WHEN_IN_FOCUSED_WINDOW);
            }
            items.put("button."+command, foo);
            return foo;
        }


        /**
         * Handy-dandy class to create menu items easily.
         *
         * @param label Text to use to label item
         * @param command Command text passed to <code>actionPerformed()</code>
         * @param mnemonic Character to use as shortcut
         * @param accel Accelerator key, eg. <code>KeyEvent.VK_S</code>
         * @return JMenuItem suitable for adding to a <var>JMenu</var>
         **/
        public JMenuItem menuItem(String label,
                                  String command,
                                  int mnemonic,
                                  int accel)
        {
            JMenuItem item = new JMenuItem(label);
            item.addActionListener(this);
            item.setActionCommand(command);
            if (mnemonic !=0) item.setMnemonic( (char)mnemonic );
            if (accel != 0) {
                int mask = Event.CTRL_MASK;
                if ((accel < KeyEvent.VK_A) || (accel > KeyEvent.VK_Z))
                    mask = 0;

                item.setAccelerator(KeyStroke.getKeyStroke(accel, mask));
            }
            items.put("menu."+command, item);
            return item;
        }


        /**
         * Useful method for creating radio button menu items.
         *
         * @param label Text to use to label item
         * @param command Command text passed back on click
         * @param group ButtonGroup to add to
         **/
        public JMenuItem menuRadioItem(String label,
                                       String command,
                                       ButtonGroup group)
        {
            JMenuItem item = new JRadioButtonMenuItem(label);
            group.add(item);
            item.addActionListener(this);
            item.setActionCommand(command);
            item.setSelected(true);

            items.put("menu."+command, item);
            return item;
        }


        /**
         * Respond to action events from all over the GUI.
         *
         * @param e Detail on what event occured
         **/
        public void actionPerformed(ActionEvent e)
        {
            String cmd     = e.getActionCommand();
            int    current = list.getSelectionModel().getMinSelectionIndex();
            int    max     = dataModel.getRowCount();
            DEBUG.println("DisplayNews::ActionDoer.actionPerformed: " + cmd +
                          " current="+current+"/"+max);

            if (cmd.equals("next") || cmd.equals("prev")) {
                // -- Display the next or previous story
                //
                current += cmd.equals("next") ? +1 : -1;
                if ((current < max) && (current >= 0)) {
                    displayStory(dataModel.getStory(current).getOriginalURL());
                    list.scrollRectToVisible( list.getCellRect(current,
                                                               1,
                                                               true)
                                              );
                }

            } else if (cmd.equals("delete") && (current < max)) {
                // -- Delete the current story
                //
                dataModel.getStory(current).delete();
                dataModel.removeInvalid();
                max = dataModel.getRowCount();
                if (current >= max) current = max - 1;
                if (current >= 0) {
                    displayStory(dataModel.getStory(current).getOriginalURL());
                } else {
                    jep.setText("");
                    setEnabled("next", false);
                    setEnabled("prev", false);
                    setEnabled("delete", false);
                    setEnabled("copy", false);
                    setEnabled("original", false);
                    setEnabled("unread", true);
                }
                list.revalidate();

            } else if (cmd.equals("unread")) {
                // -- Mark the current story as unread
                //
                dataModel.getStory(current).setIsRead(Boolean.FALSE);
                list.repaint();

            } else if (cmd.equals("gather")) {
                // -- Collect the news
                //
                Thread gather = new Thread(new Runnable() {
                        public void run() {
                            GetNews news = new GetNews();
                            news.main();
                            JOptionPane.showMessageDialog(win,
                                                          "Fetch completed.");
                            setEnabled("gather", true);
                            dataModel.readCache();
                            list.revalidate();
                            return;
                        }
                    });
                setEnabled("gather", false);
                dataModel.writeBack();
                gather.start();

            } else if (cmd.equals("showall") || cmd.equals("shownew")) {
                // -- Change the list of stories currently displayed
                //
                dataModel.writeBack();
                dataModel.readCache( (cmd.equals("showall") ? CacheLink.ALL :
                                                              CacheLink.UNREAD) );
                list.revalidate();

            } else if (cmd.equals("options")) {
                // -- Open the misc. options window
                //
                miscConf.setVisible(true);

            } else if (cmd.equals("plugins")) {
                // -- Open the plugin config window
                //
                pluginConf.setVisible(true);

            } else if (cmd.equals("copy")) {
                // -- Copy any selected text to the system clipboard
                //
                jep.copy();

            } else if (cmd.equals("select")) {
                // -- Select all the text in the mini-browser
                //
                jep.selectAll();

            } else if (cmd.equals("original")) {
                // -- Open the original page for this story
                //
                openExternal(dataModel.getStory(current).getOriginalURL());

            } else if (cmd.equals("about")) {
                // -- Display the application's "About" box
                //
                String[] msg = { "Paperboy",
                                 "Collection, Processing and Presentation of Online News",
                                 " ",
                                 "(c) Andrew Flegg 2000, v"+Paperboy.VERSION,
                                 "http://www.bleb.org/software/paperboy/" };
                JOptionPane.showMessageDialog(tlp, msg, "About Paperboy...",
                                              JOptionPane.INFORMATION_MESSAGE);

            } else if (cmd.equals("web")) {
                // -- Open the home page for Paperboy
                //
                openExternal("http://www.eh.org/~jaffa/paperboy/");
                
            } else if (cmd.equals("exit")) {
                // -- Quit the application
                //
                win.setVisible(false);
                dataModel.writeBack();
                System.exit(0);
            }
                
            return;
        }


        /**
         * Enable/disable a button and/or a menu item.
         *
         * @param name command to alter
         * @param val true to enable, false otherwise
         **/
        public void setEnabled(String name, boolean val)
        {
            JComponent ctrl;
            ctrl = (JComponent)items.get("button."+name);
            if (ctrl != null) {
                ctrl.setEnabled(val);
                ((JButton)ctrl).setSelected(false);
            }

            ctrl = (JComponent)items.get("menu."+name);
            if (ctrl != null) ctrl.setEnabled(val);
            return;
        }
    }


    // -----------------------------------------------------------------


    /**
     * Implement an interface to the cache for the GUI
     *
     * @version 0.00 (04-Mar-2000);
     **/
    public class CacheLink extends AbstractTableModel
    {
        private final String[] columns = { "New", "Site", "Headline", "Date" };

        private int     lastType;
        private Story[] storyList;
        public static final int ALL    = 0;
        public static final int UNREAD = 1;


        /**
         * Default constructor calls <code>readCache()</code>
         **/
        public CacheLink()
        {
            super();
            lastType = UNREAD;
            readCache();
        }


        /**
         * Ensure all cache data is rewritten.
         **/
        public void writeBack()
        {
            for (int i=0; i<storyList.length; i++)
                storyList[i].updateHash();
            Cache.writeUsage();

            return;
        }


        /**
         * Read the data from the cache
         **/
        public void readCache()
        {
            readCache(lastType);
            return;
        }


        /**
         * Remove invalid, ie. deleted, entries from the table
         **/
        public void removeInvalid()
        {
            Vector v = new Vector();
            for (int i=0; i<storyList.length; i++) {
                Story s = storyList[i];
                if (Cache.stored(s.getOriginalURL())) v.add(s);
            }

            storyList = (Story[])v.toArray(new Story[0]);
            DEBUG.println("CacheLink.removeInvalid:  " + storyList.length +
                          " article(s) found.");
            return;
        }


        /**
         * Read specific data from the cache. If
         * <var>CacheLink.ALL</var> is passed then all the cached
         * Story objects are fetched, otherwise only the unread
         * ones.
         *
         * @param type which stories to read in
         **/
        public void readCache(int type)
        {
            // -- Read the cache, and get the Story objects
            DEBUG.print("CacheLink.readCache: ");
            Object[] obj = Cache.list("html").toArray();
            storyList    = new Story[obj.length];
            for (int i=0; i<obj.length; i++)
                storyList[i] = (Story)Cache.get( (String)obj[i] );

            // -- Limit the stories if appropriate
            if (type == UNREAD) {
                Vector sList = new Vector();
                for (int i=0; i<storyList.length; i++)
                    if (! storyList[i].isRead()) sList.add(storyList[i]);

                storyList = (Story[])sList.toArray(new Story[0]);
            }
            DEBUG.println(storyList.length + " article(s) found.");

            // -- Sort them into order: by section and then by
            //    headline, eg. site:section headline...
            Comparator c = new Comparator() {
                    public int compare(Object o1, Object o2) {
                        Story  s1 = (Story)o1;
                        Story  s2 = (Story)o2;
                        String k1 = s1.getSection() + s1.getHeadline();
                        String k2 = s2.getSection() + s2.getHeadline();
                        return k1.compareTo(k2);
                    }
                };
            Arrays.sort(storyList, c);

            // -- Update the record of what we're looking at ATM
            lastType = type;
            return;
        }

         
        /**
         * Get a column name
         *
         * @param c column number (from 0)
         * @return Name of that column
         **/
        public String getColumnName(int c) { return columns[c]; }


        /**
         * Get the number of columns
         *
         * @return Number of columns in the table
         **/
        public int getColumnCount() { return columns.length; }


        /**
         * Get the number of rows
         *
         * @return Number of rows in the table
         **/
        public int getRowCount() { return storyList.length; }


        /**
         * Get the story for the given row.
         *
         * @param row story to return
         * @return Story for that row
         **/
        public Story getStory(int row) { return storyList[row]; }


        /**
         * Get the index of a Story from the original URL
         *
         * @param u original URL for the Story
         * @return Story row number
         **/
        public int getStoryIndex(String u)
        {
            int s = -1;
            for (int i=0; i<storyList.length; i++) {
                if (storyList[i].getOriginalURL().equals(u)) {
                    s = i;
                    break;
                }
            }
            return s;
        }


        /**
         * Find type of data in a particular column
         *
         * @param col field to return (0...)
         * @return Class at that position
         **/
        public Class getColumnClass(int col)
        {
            String obj = columns[col];
            Class  val = String.class;

            // -- Safety checking
            if (obj == null) return String.class;

            if (obj.equals("New")) {
                val = Boolean.class;
                
            } else if (obj.equals("Date")) {
                val = Date.class;
            }

            return val;
        }


        /**
         * Get data at a particular position in the table.
         *
         * @param row story to return (0...)
         * @param col field to return (0...)
         * @return Data at that position
         **/
        public Object getValueAt(int row, int col)
        {
            String  obj    = columns[col];
            Object  val    = null;
            Story   s      = storyList[row];

            // -- Safety checking
            if ((s == null) || (obj == null)) return null;

            if (obj.equals("Headline")) {
                val = s.getHeadline();
                
            } else if (obj.equals("Site")) {
                val = s.getSection();

            } else if (obj.equals("Date")) {
                val = new Date(s.getLastModified());

            } else if (obj.equals("URL")) {
                val = s.getOriginalURL();

            } else if (obj.equals("New")) {
                val = new Boolean(! s.isRead());
            }

            return val;
        }
    }
}
