package org.bleb.paperboy.gui;

import org.bleb.paperboy.plugin.*;
import org.bleb.paperboy.util.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.border.Border;
import java.util.*;

/**
 * Display a configuration window allowing the options for each
 * plugin to be changed individually.
 *
 * @version 0.01 (08-Mar-2000)
 **/
public class PluginConfigFrame implements ActionListener
{
    protected JFrame           window;
    protected JTabbedPane      pane;
    protected JButton          save;
    protected JButton          cancel;
    protected JButton          rules;
    protected RulesConfigFrame rulesConf;

    public PluginConfigFrame()
    {
        Border bigGap = BorderFactory.createEmptyBorder(12, 12, 12, 12);
        Border midGap = BorderFactory.createEmptyBorder(8, 8, 8, 8);
        int    skip   = new String("Plugin").length();

        rulesConf = new RulesConfigFrame();
        window    = new JFrame("Plugin Configuration");
        pane      = new JTabbedPane();
        pane.setBorder(midGap);

        JPanel box = new JPanel();
        box.setLayout(new BoxLayout(box, BoxLayout.Y_AXIS));
        box.setBorder(midGap);

        save   = new JButton("Save");
        cancel = new JButton("Close");
        rules  = new JButton("Rules...");

        save.addActionListener(this);
        cancel.addActionListener(this);
        rules.addActionListener(this);

        save.setActionCommand("save");
        cancel.setActionCommand("cancel");
        rules.setActionCommand("rules");

        save.setMaximumSize(rules.getPreferredSize());
        cancel.setMaximumSize(rules.getPreferredSize());
        rules.setMaximumSize(rules.getPreferredSize());

        box.add(Box.createVerticalGlue());
        box.add(save);
        box.add(Box.createVerticalStrut(15));
        box.add(cancel);
        box.add(Box.createVerticalStrut(15));
        box.add(rules);

        Enumeration list = Plugin.list();
        while (list.hasMoreElements()) {
            String       name = (String)list.nextElement();
            OptionConfig o    = Plugin.getConfig(name);
            JScrollPane  view = new JScrollPane(o.getComponent());
            view.setBorder(bigGap);
            pane.addTab(name.substring(skip), view);
        }

        window.getContentPane().add(pane, "Center");
        window.getContentPane().add(box, "East");
        window.setSize(512, 400);
    }

    public void setVisible(boolean b)
    {
        window.setVisible(b);
        return;
    }


    public void actionPerformed(ActionEvent e)
    {
        String cmd     = e.getActionCommand();
        String current = "Plugin" + pane.getTitleAt(pane.getSelectedIndex());

        if (cmd.equals("save")) {
            // -- Save options for *current plugin only*
            //
            DEBUG.print("PluginConfigFrame.actionPerformed: Saving " +
                        current + "... ");
            FileOutput.Line out = new FileOutput.Line(current+".conf");
            if (out.open()) {
                out.print( Plugin.getConfig(current).toString() );
                out.close();
                DEBUG.println("complete.");
            } else {
                DEBUG.println("failed.");
            }

        } else if (cmd.equals("cancel")) {
            // -- Cancel changes and close window
            //
            window.setVisible(false);

        } else if (cmd.equals("rules")) {
            // -- Open the rules for this plugin
            //
            rulesConf.openConfig(current);
        }
        return;
    }
}





