package org.bleb.paperboy.gui;

import org.bleb.paperboy.rule.*;
import org.bleb.paperboy.plugin.*;
import org.bleb.paperboy.util.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.util.*;

/**
 * Display a configuration window allowing the rules for a
 * plugin to be changed.
 *
 * @version 0.01 (09-Mar-2000)
 **/
public class RulesConfigFrame implements ActionListener
{
    protected JFrame        window;
    protected JPanel        pane;
    protected JScrollPane   content;
    protected JButton       save;
    protected JButton       cancel;
    protected JList         listView;

    protected JButton       edit;
    protected JButton       add;
    protected JButton       delete;

    protected Rule[]        list;
    protected String[]      listText;
    protected int           skip;
    protected String        plugin;

    protected Border        bigGap;
    protected Border        midGap;

    public RulesConfigFrame()
    {
        super();
        bigGap    = BorderFactory.createEmptyBorder(12, 12, 12, 12);
        midGap    = BorderFactory.createEmptyBorder(8, 8, 8, 8);

        skip      = new String("Plugin").length();
        window    = new JFrame("Rules");
        content   = new JScrollPane();
        pane      = new JPanel();
        pane.setBorder(new CompoundBorder(bigGap,
                                          BorderFactory.createRaisedBevelBorder()));
        content.setPreferredSize( new Dimension(300, 220) );
        content.setBorder(BorderFactory.createLoweredBevelBorder());
        content.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        content.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        JPanel boxBot = new JPanel();
        boxBot.setLayout(new BoxLayout(boxBot, BoxLayout.X_AXIS));
        boxBot.setBorder(midGap);

        JPanel boxSide = new JPanel();
        boxSide.setLayout(new BoxLayout(boxSide, BoxLayout.Y_AXIS));
        boxSide.setBorder(midGap);

        save   = new JButton("Save");
        cancel = new JButton("Close");
        edit   = new JButton("Edit");
        add    = new JButton("Add");
        delete = new JButton("Delete");

        save.addActionListener(this);
        cancel.addActionListener(this);
        edit.addActionListener(this);
        add.addActionListener(this);
        delete.addActionListener(this);

        save.setActionCommand("save");
        cancel.setActionCommand("cancel");
        edit.setActionCommand("edit");
        add.setActionCommand("add");
        delete.setActionCommand("delete");

        save.setMaximumSize(cancel.getPreferredSize());
        cancel.setMaximumSize(cancel.getPreferredSize());
        edit.setMaximumSize(delete.getPreferredSize());
        add.setMaximumSize(delete.getPreferredSize());
        delete.setMaximumSize(delete.getPreferredSize());

        boxBot.add(Box.createHorizontalGlue());
        boxBot.add(save);
        boxBot.add(Box.createHorizontalStrut(15));
        boxBot.add(cancel);
        boxBot.add(Box.createHorizontalGlue());

        boxSide.add(Box.createVerticalGlue());
        boxSide.add(add);
        boxSide.add(Box.createVerticalStrut(15));
        boxSide.add(edit);
        boxSide.add(Box.createVerticalStrut(15));
        boxSide.add(delete);

        pane.add(content, "Center");
        pane.add(boxSide, "East");
        window.getContentPane().add(pane, "Center");
        window.getContentPane().add(boxBot, "South");
        window.setSize(464, 320);
    }


    /**
     * Open the configuration window for the specified plugin
     *
     * @param name Name of the plugin, eg. <code>PluginBBC</code>
     **/
    public void openConfig(String name)
    {
        plugin   = name;
        listView = new JList();
        listView.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        setList(Rule.getList(name));
        if (list.length > 0) {
            listView.setSelectedIndex(0);
            edit.setEnabled(true);
            delete.setEnabled(true);
        } else {
            edit.setEnabled(false);
            delete.setEnabled(false);
        }

        content.setViewportView(listView);
        window.setTitle(name.substring(skip) + " Rules");
        window.setVisible(true);
        return;
    }


    /**
     * Turn a list of rules into an array of strings
     *
     * @param l Rules to convert
     * @return a String representation of the list
     **/
    protected String[] getListText(Rule[] l)
    {
        String[] text;

        if (l == null) {
            text = new String[0];
        } else {
            text = new String[l.length];
            for (int i=0; i<l.length; i++)
                text[i] = l[i].toString();
        }
        return text;
    }


    /**
     * Update the internal list representations.
     *
     * @param l List of Rules to set as
     **/
    protected void setList(Rule[] l)
    {
        if (l == null) l = new Rule[0];

        list     = l;
        listText = getListText(l);
        listView.setListData(listText);
        return;
    }


    // -- Implements ActionListener
    //
    public void actionPerformed(ActionEvent e)
    {
        String cmd = e.getActionCommand();

        if (cmd.equals("save")) {
            // -- Save the updated rules for this plugin
            //
            Rule.setList(plugin, list);
            Rule.writeBack();
            window.setVisible(false);

        } else if (cmd.equals("cancel")) {
            // -- Cancel changes and close window
            //
            window.setVisible(false);

        } else if (cmd.equals("edit")) {
            // -- Edit selected rule
            //
            String rule = JOptionPane.showInputDialog(window,
                                                      "Please enter replacement rule");
            if (rule == null) return;
            try {
                Rule r = Rule.getRule(rule);
                DEBUG.println("RulesConfigFrame.actionPerformed: Edited rule '" +
                              r.toString() + "'");

                int pos = listView.getSelectedIndex();
                list[pos] = r;
                setList(list);
                listView.setSelectedIndex(pos);
            } catch (ParseException pe) {
                JOptionPane.showMessageDialog(window, new String[] {
                    "Syntax error in rule:",
                    "    " + rule,
                    " ",
                    "Problem: " + pe.getMessage() });
            }

        } else if (cmd.equals("add")) {
            // -- Add a new rule
            //
            String rule = JOptionPane.showInputDialog(window, "Please enter new rule");
            if (rule == null) return;

            try {
                Rule r = Rule.getRule(rule);
                DEBUG.println("RulesConfigFrame.actionPerformed: New rule '" +
                              r.toString() + "'");

                Rule[] temp = new Rule[list.length + 1];
                for (int i=0; i<list.length; i++) temp[i] = list[i];
                temp[list.length] = r;
                setList(temp);

                listView.setSelectedIndex(list.length - 1);
                edit.setEnabled(true);
                delete.setEnabled(true);
            } catch (ParseException pe) {
                JOptionPane.showMessageDialog(window, new String[] {
                    "Syntax error in rule:",
                    "    " + rule,
                    " ",
                    "Problem: " + pe.getMessage() });
            }

        } else if (cmd.equals("delete")) {
            // -- Delete selected rule
            //
            Rule[] temp = new Rule[list.length - 1];
            int    next = 0;
            for (int i=0; i<list.length; i++) {
                if (i != listView.getSelectedIndex()) temp[next++] = list[i];
            }
            setList(temp);

            if (list.length > 0) {
                listView.setSelectedIndex(0);
            } else {
                edit.setEnabled(false);
                delete.setEnabled(false);
            }
        }
        return;
    }
}





