package org.bleb.paperboy.gui.htmlview;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import org.bleb.paperboy.gui.*;
import org.bleb.paperboy.util.*;

/**
 * A hyperlink listener for use with JEditorPane.  This
 * listener will change the cursor over hotspots based on enter/exit
 * events and also load a new page when a valid hyperlink is clicked.
 *
 * @version 0.02 (01-Mar-2000)
 **/
public class SimpleLinkListener implements HyperlinkListener {

    private JEditorPane pane;       // The pane we're using to display HTML

    private JTextField  urlField;   // An optional textfield for showing
                                    // the current URL being displayed

    private JLabel statusBar;       // An optional label for showing where
                                    // a link would take you

    public String oldURL;

    /**
     * Create a new listener and register the visible objects.
     *
     * @param jep Pane being used to display the HTML
     * @param jtf Optional textfield displaying current URL
     * @param jl  Optional label for display links' locations
     **/
    public SimpleLinkListener(JEditorPane jep, JTextField jtf, JLabel jl)
    {
        pane      = jep;
        urlField  = jtf;
        statusBar = jl;
    }


    /**
     * Create a listener for just displaying HTML.
     *
     * @param jep Pane being used to display the HTML
     **/
    public SimpleLinkListener(JEditorPane jep)
    {
        this(jep, null, null);
    }


    /**
     * Hyperlink event listener
     *
     * @param he Details of the event
     **/
    public void hyperlinkUpdate(HyperlinkEvent he)
    {
        HyperlinkEvent.EventType type = he.getEventType();

        if (type == HyperlinkEvent.EventType.ENTERED) {
            // -- Link entered, change mouse pointer and update status bar
            //
            pane.setCursor( Cursor.getPredefinedCursor(Cursor.HAND_CURSOR) );
            setStatus(he.getURL().toString());
            
        } else if (type == HyperlinkEvent.EventType.EXITED) {
            // -- Link exited, change pointer back and blank status bar
            //
            pane.setCursor( Cursor.getDefaultCursor() );
            setStatus(" ");
            
        } else {
            // -- Link activated, change pane and update URL box
            //
            Cursor old = pane.getCursor();
            pane.setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR) );

            String       u    = he.getURL().toString();
            StoryDisplay disp =
                (StoryDisplay)pane.getClientProperty("paperboy.display");
            if (disp != null) {
                disp.displayStory(u);
            } else {
                try { pane.setPage(u); } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            pane.setCursor(old);
        }
        return;
    }
    
    
    // -- Update the status bar if set
    //
    private void setStatus(String msg)
    {
        if (statusBar != null) statusBar.setText(msg);
        return;
    }
}
