package org.bleb.paperboy.gui.htmlview;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.event.*;
import java.awt.*;

/**
 * A test bed for the JEditorPane and a custom editor kit.
 * This extremely simple browser has a text field for typing in
 * new urls, a JEditorPane to display the HTML page, and a status
 * bar to display the contents of hyperlinks the mouse passes over.
 *
 * @version 0.00 (05-Feb-1999)
 **/
public class TestBrowser extends JFrame
{
    protected SimpleLinkListener sll;
    protected JTextField urlField;

    /**
     * Test the web browser - demonstrate the technology.
     *
     * @param startingUrl String containing URI of initial page.
     **/
    public TestBrowser(String startingUrl)
    {
        // -- Create window and register close window command
        //
        super("Test Browser");
        setSize(520, 512);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent we)
            {
                    we.getWindow().setVisible(false);
                    System.exit(0);
            }
        });

        // -- Set up gui components (main pane, URL text field and
        //    status/link labels
        //
        JPanel urlPanel = new JPanel();
        urlPanel.setLayout(new BorderLayout());
        urlField = new JTextField(startingUrl);
        JButton urlBack = new JButton("Back");
        urlPanel.add(new JLabel("Site: "), BorderLayout.WEST);
        urlPanel.add(urlField, BorderLayout.CENTER);
        urlPanel.add(urlBack, BorderLayout.EAST);
        final JLabel statusBar = new JLabel(" ");

        // -- Configure the editor pane, setEditable(false) so hyperlinks
        //    work (if text is editable then clicking link changes the text)
        //
        final JEditorPane jep = new JEditorPane();
        jep.setEditable(false);
        jep.setEditorKitForContentType("text/html", new PatchedHTMLEditorKit());


        // -- Fetch start page
        //
        try {
            jep.setPage(startingUrl);
        } catch(Exception e) {
            statusBar.setText("Could not open starting page. Using a blank.");
        }
        
        // -- Add components to window
        //
        JScrollPane jsp = new JScrollPane(jep); 
        getContentPane().add(jsp, BorderLayout.CENTER);
        getContentPane().add(urlPanel, BorderLayout.NORTH);
        getContentPane().add(statusBar, BorderLayout.SOUTH);

        // -- Register other event handlers
        //
        urlField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae)
            {
                    try {
                        jep.setPage(ae.getActionCommand());
                    } catch(Exception e) {
                        statusBar.setText("Could not open page. Using a blank.");
                    }
            }
        });

        sll = new SimpleLinkListener(jep, urlField, statusBar);

        urlBack.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae)
            {
                try {
                    jep.setPage(sll.oldURL);
                    urlField.setText(sll.oldURL);
                } catch (Exception e) {
                    statusBar.setText("Error going back...");
                }
            }
        });

        jep.addHyperlinkListener(sll);
    }


    /**
     * Simply create an instance of the app and open it.
     *
     **/
    public static void main(String args[])
    {
        TestBrowser app = new TestBrowser((args.length > 0)
                                                  ? args[0] 
                                                  : "file:///c:/");
        app.setVisible(true);
    }
}
