package org.bleb.paperboy.plugin;

import javax.swing.JComponent;

/**
 * Interface implemented by the types of option, eg. boolean, text,
 * textarea etc.
 *
 * @version 0.01 (08-Mar-2000)
 **/
public abstract class Option implements Drawable
{
    private String name;
    private String label;
    private static boolean createGUI = false;

    public Option()
    {
        super();
        name  = "";
        label = "";
    }

    /**
     * Set whether or not to create the GUI components.
     * Default: <var>false</var>
     *
     * @param b Set GUI creation
     **/
    public static void setCreateGUI(boolean b)
    {
        createGUI = b;
    }


    /**
     * Should the GUI components be created?
     *
     * @return true if the GUI should be created
     **/
    public static boolean getCreateGUI()
    {
        return createGUI;
    }


    /**
     * Get the object containing this fields data
     *
     * @return Object holding value
     **/
    public abstract Object getValue();


    /**
     * Set this field's value exactly.
     *
     * @param o new value
     **/
    public abstract void setValue(Object o);


    /**
     * Interpret a string for the value.
     *
     * @param s String containing value, eg. <code>true</code>
     **/
    public abstract void setValue(String o);


    /**
     * Get the type of this field
     *
     * @return Value used in <code>type="..."</code>
     **/
    public abstract String getType();

    /**
     * Print yourself.
     *
     * @return String representation of this option's state
     **/
    public String toString()
    {
        return "<option type=\"" + getType() + "\" name=\"" +
            getName() + "\" label=\"" + getLabel() + "\">" +
            getValue() + "</option>";
    }


    /**
     * Get this field's name
     *
     * @return This field's name
     **/
    public String getName()
    {
        return name;
    }


    /**
     * Set this field's name
     *
     * @param n New name
     **/
    public void setName(String n)
    {
        name = n;
    }


    /**
     * Get this field's label
     *
     * @return This field's label
     **/
    public String getLabel()
    {
        return label;
    }


    /**
     * Set this field's label
     *
     * @param l New label
     **/
    public void setLabel(String l)
    {
        label = l;
    }


    // -- Implements Drawable
    public abstract JComponent getComponent();
}



