package org.bleb.paperboy.plugin;

import org.bleb.paperboy.util.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;

/**
 * Overall container for the configuration
 *
 * @version 0.01 (08-Mar-2000)
 **/
public class OptionConfig extends OptionGroup
{
    private String label;

    // -- Reimplements Option
    //
    public String getType() { return "config"; }
    public String getLabel() { return label; }
    public void setLabel(String l) { label = l; }

    public String toString() {
        String      out  = "<config app=\"Paperboy\">\n";
        Enumeration enum = items.elements();
        while (enum.hasMoreElements()) {
            out += ( (Option)enum.nextElement() ).toString() + "\n";
        }
        out += "</config>";
        return out;
    }


    /**
     * Loads an option tree from the specified file.
     * If the file cannot be accessed <var>null</var> will
     * be returned.
     *
     * @param file Filename to use
     **/
    public static OptionConfig getFile(String file)
    {
        OptionConfig   out  = null;
        FileInput.Line in   = new FileInput.Line(file);
        String         line = null;

        if (!in.open()) return null;
        while ((! in.eof()) && (out == null)) {
            if (readln(in).startsWith("<config")) out = readConfig(in);
        }
        in.close();
        return out;
    }


    // -- Read in a CONFIG block
    //
    private static OptionConfig readConfig(FileInput.Line in)
    {
        OptionConfig out = new OptionConfig();
        String line      = null;

        while (! (line = readln(in)).equals("</config>")) {
            Option toAdd = null;
            if (line.startsWith("<group") ||
                line.startsWith("<option")) {
                toAdd = readSection(line, in);
            }
            if (toAdd != null) out.add(toAdd);
        }
        return out;
    }


    // -- Read in a GROUP block
    //
    private static Option readSection(String line, FileInput.Line in)
    {
        Properties  tag     = tagData(line);
        String      objName = tag.getProperty("tag.type");
        Option      out     = null;

        // -- Get the class name
        if (objName.equals("group")) {
            objName = "Group";
        } else if (objName.equals("option")) {
            objName = tag.getProperty("type");
            objName = objName.substring(0,1).toUpperCase() +
                objName.substring(1);
        }

        // -- Try instantiating the class
        try {
            objName = "org.bleb.paperboy.plugin.Option" + objName;
            out     = (Option)Class.forName(objName).newInstance();
        } catch(Exception e) {
            DEBUG.println("OptionConfig.readSection: Bad type - " + objName);
            return null;
        }

        // -- Set common data
        out.setName( tag.getProperty("name") );
        out.setLabel( tag.getProperty("label") );

        if (out instanceof OptionGroup) {
            // -- Recurse into a subgrouop
            //
            while (! (line = readln(in)).equals("</group>")) {
                Option toAdd = null;
                if (line.startsWith("<group") ||
                    line.startsWith("<option")) {
                    toAdd = readSection(line, in);
                }
                if (toAdd != null) ((OptionGroup)out).add(toAdd);
            }

        } else {
            // -- Just an option
            //
            out.setValue( tag.getProperty("tag.data") );
            if (tag.getProperty("tag.closed") == null)
                DEBUG.println("WARNING: Tag isn't closed...");
        }
        return out;
    }


    // -- Read a line in the correct format
    //
    private static String readln(FileInput.Line in)
    {
        String line = in.readln().trim();
        if ((line == null) || line.startsWith("//")) return "";

        return line;
    }


    // -- Convert a line to a set of tag data
    //
    private static Properties tagData(String line)
    {
        Properties out   = new Properties();
        int        apos  = line.indexOf(">");
        String     open  = line.substring(1, apos);
        String     data  = line.substring(apos + 1);
        int        cpos  = data.indexOf("</");

        // Have we got a close tag?
        if (cpos != -1) {
            data = data.substring(0, cpos);
            out.setProperty("tag.closed", "true");
            out.setProperty("tag.data", data);
        }

        // Find the tag name
        apos = open.indexOf(" ");
        out.setProperty("tag.type", open.substring(0, apos));
        open = open.substring(apos + 1);

        // Parse the rest using a tokeniser
        StringTokenizer tkn  = new StringTokenizer(open, "\"");
        while (tkn.hasMoreTokens()) {
            String var = tkn.nextToken().trim();
            String val = tkn.nextToken().trim();
            var = var.substring(0, var.length() - 1);
            out.setProperty(var, val);
        }

        return out;
    }
}



