package org.bleb.paperboy.plugin;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * Represents a collection of Options
 *
 * @version 0.01 (08-Mar-2000)
 **/
public class OptionGroup extends Option
    implements Drawable
{
    protected JPanel box;
    protected Vector items;


    /**
     * Create new Group and specify its label
     **/
    public OptionGroup()
    {
        super();
        items = new Vector();
        if (getCreateGUI()) {
            box   = new JPanel();
            box.setLayout(new BoxLayout(box, BoxLayout.Y_AXIS));
        } else {
            box = null;
        }
    }


    /**
     * Add a new option to the group
     *
     * @param o New option to add
     **/
    public void add(Option o)
    {
        if (box != null)
            box.add(o.getComponent());
        items.add(o);
        return;
    }


    // -- Implements Option
    //
    public String getType() { return "group"; }
    public void setValue(String s) { return; } // Not implemented
    public void setValue(Object o)
    {
        items = (Vector)o;
    }
    public Object getValue()
    {
        return items;
    }
    public void setLabel(String l)
    {
        super.setLabel(l);
        if (box != null)
            box.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(),
                                           l));
        return;
    }


    /**
     * Return a particular item from the group
     *
     * @param name Name of item in this group
     * @return Particular option
     **/
    public Option getItem(String name)
    {
        Option      found = null;
        Enumeration enum  = items.elements();
        while (enum.hasMoreElements() && (found == null)) {
            Option current = (Option)enum.nextElement();
            if (current.getName().equals(name)) found = current;
        }
        return found;
    }


    // -- For output purposes
    //
    public String toString()
    {
        String      out  = "<group name=\"" + getName() +
                           "\" label=\"" + getLabel() + "\">\n";
        Enumeration enum = items.elements();
        while (enum.hasMoreElements()) {
            out += ( (Option)enum.nextElement() ).toString() + "\n";
        }
        out += "</group>";
        return out;
    }
        

    // -- Implements Drawable
    //
    public JComponent getComponent()
    {
        return box;
    }
}





