package org.bleb.paperboy.rule;

import java.util.*;

/**
 * A simple part of the tree
 *
 * @version 1.01 (25-Feb-1999)
 **/
public class RuleNode extends Rule
{
    private String value;

    
    /**
     * A simple constructor for a simple node.
     *
     * @param v String containing text for leaf
     **/
    public RuleNode(String v)
    {
        super();
        value = v;
    }
    
    
    public String toString()
    {
        String ret = value;
        
        if (value.indexOf(" ") > 0) { ret = "\"" + value + "\""; }
        return ret;
    }
    
    
    public boolean matches(String text, RuleConfig c)
    {
        String v = (c.isCaseSensitive() ? value : value.toLowerCase());
        
        return (text.indexOf(v) != -1);
    }
    
    
    /**
     * Finds all occurences of this text within a string. Returns a 
     * Vector containing the word count of each instance.
     *
     * @param text String containing block of text to search
     * @param cfg configuration information (case sensitive or not?)
     * @return Vector of all integer positions
     **/
    public Vector positions(String text, RuleConfig cfg)
    {
        Vector  v         = new Vector();    // will store word counts
        int     word      = 1;               // count of current word
        int     end       = text.length();   // end of text
        boolean newWord   = true;            // if set then don't increment word
        int     lenSearch = value.length();  // length of word looking for
        char    c;                           // current character
        String  val       = (cfg.isCaseSensitive() ? value : value.toLowerCase());

        for (int pos = 0; pos < end; pos++) {
            c = text.charAt(pos);

            // -- If a space then increment word count, otherwise check
            //    if we can match
            //
            if ((c == ' ') || (c == '\t') || (c == '\n') || (c == '\r')) {
                if (! newWord) { word++; newWord = true; }
            } else {
                try {
                    if (text.substring(pos, pos + lenSearch).startsWith(val)) {
                        v.add(new Integer(word));
                        pos += lenSearch - 1;    // skip over rest of word
                    }
                } catch (StringIndexOutOfBoundsException e) {
                    // Didn't match
                }
                newWord = false;
            }
        }
        
        return v;
    }
}
