package org.bleb.paperboy.util;

import java.util.*;
import org.bleb.paperboy.plugin.*;

/**
 * Manage the non-plugin specific options, eg. proxy & external
 * viewer.
 *
 * @version 0.01 (08-Mar-2000)
 **/
public class Config extends Object
{
    protected static OptionConfig conf;

    /**
     * Initialise the miscellaneous options, should be called
     * <em>before</em> <code>Plugin.init()</code> and
     * <code>Cache.init()</code>.
     **/
    public static void init()
    {
        String dir1 = System.getProperty("paperboy.data");
        String dir2 = System.getProperty("paperboy.lib");

        FileInput.setRoot(dir1);
        conf = OptionConfig.getFile("paperboy.conf");

        if (conf == null) {
            FileInput.setRoot(dir2);
            conf = OptionConfig.getFile("paperboy.conf");

            if (conf == null) {
                conf = new OptionConfig();
                DEBUG.println("Config.init: Can't find paperboy.conf");
            }
        }
        FileInput.setRoot(".");
        DEBUG.println("Config.init: Complete.");
        return;
    }


    /**
     * Get the configuration.
     *
     * @return Configuration options as specified in <code>paperboy.conf</code>
     **/
    public static OptionConfig getConfig()
    {
        if (conf == null) DEBUG.println("Config.getConfig: Called before init()!");
        return conf;
    }


    /**
     * Get the proxy host.
     *
     * @return Hostname of HTTP proxy
     **/
    public static String getProxyHost()
    {
        String addr = (String)conf.getItem("proxy").getValue();
        int    pos  = addr.indexOf(":");

        if (pos == -1) return addr;
        return addr.substring(0, pos);
    }


    /**
     * Get the proxy port.
     *
     * @return Port number of HTTP proxy
     **/
    public static int getProxyPort()
    {
        String addr = (String)conf.getItem("proxy").getValue();
        int    pos  = addr.indexOf(":");

        if (pos == -1) return 0;
        return (new Integer(addr.substring(pos+1))).intValue();
    }


    /**
     * Get the external viewer command for a particular URL
     *
     * @param u URL to view
     * @return Command to execute to show URL
     **/
    public static String getViewerCommand(String u)
    {
        String cmd = (String)conf.getItem("viewer").getValue();
        int    pos = cmd.indexOf("%u");

        if (pos == -1) return cmd;
        return cmd.substring(0, pos) + u + cmd.substring(pos+2);
    }
}

