package org.bleb.paperboy.util;

import java.io.*;

public class DEBUG
{
    private static boolean writing = true;
    private static PrintStream out = System.err;

    
    /**
     * Print a line to the current output if debug is enabled.
     *
     * @param s Object to pass to the <var>PrintStream</var>
     **/
    public static void print(String s)
    {
        if (writing) out.print(s);
        return;
    }
    
    public static void println(String s)
    {
        if (writing) out.println(s);
        return;
    }
    

    /**
     * Set whether or not debug message are written.
     *
     * @param w true if debug messages should be written
     **/
    public static void setState(boolean w)
    {
        writing = w;
        return;
    }
    
    
    /**
     * Return the state of the debug.
     *
     * @return true if debug messages are written
     **/
    public static boolean getState() { return writing; }
    
    
    /**
     * Set the output stream to send debug messages to.
     * The following code fragment will set one up:
     * <pre>
     * PrinStream p = System.err;
     * try {
     *     p = new PrintStream(new FileOutputStream("debug.txt"));
     * } catch (FileNotFoundException e) {
     *     p = System.err;
     * }
     * DEBUG.setStream(p);</pre>
     *
     * @param p PrintStream object
     **/
    public static void setStream(PrintStream p) { out = p; }
    
    
    /**
     * Return the PrintStream currently being used
     *
     * @return PrintStream used for debug messages
     **/
    public static PrintStream getStream() { return out; }
}
